==> qfsm-0.43.tar.gz: qfsm-0.43/src/ExportVHDL.cpp <==
/*
Copyright (C) 2000,2001 Stefan Duffner 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <iostream>
#include <qregexp.h>

#include "ExportVHDL.h"
#include "Machine.h"
#include "TransitionInfo.h"
#include "IOInfo.h"
#include "IOInfoList.h"
#include "Convert.h"
#include "Utils.h"
#include "Options.h"

using namespace std;


ExportVHDL::ExportVHDL(Options* opt)
  	  : Export(opt)
{
}


/// Writes all the relevant data into the tdf file.
void ExportVHDL::doExport()
{
  state_encoding=!options->getVHDLSymbolicStates();
  synchronous_reset=options->getVHDLSyncReset();
  use_std_logic=options->getVHDLStdLogic();

  writeHeader("--", "");
  writeEnvironment();
  writeEntity();
  writeArchitecture();
}


QString ExportVHDL::fileFilter()
{
  return "VHDL (*.vhdl)";
}

/// Writes the 'library' and 'use' lines to the output stream
void ExportVHDL::writeEnvironment()
{
  if (use_std_logic)
  {
    *out << "library ieee;" << endl << endl;
    *out << "use ieee.std_logic_1164.all;" << endl << endl;
  }
}

/// Writes the 'entity' block to the output stream
void ExportVHDL::writeEntity()
{
  QString bit_string;

  if (use_std_logic)
    bit_string = "std_logic";
  else
    bit_string = "bit";

  mname = machine->getName();
  mname.replace(QRegExp(" "), "_");

  *out << "entity " << mname << " is" << endl;
  *out << "--  generic (prop_delay : Time := 10 ns);" << endl;
  *out << "  port (clock: in " << bit_string << ";" << endl;
  *out << "        reset: in " << bit_string << ";" << endl;
  *out << "        a: in " << bit_string << "_vector(" << machine->getNumInputs()-1 
    << " downto 0);" << endl;
  if (machine->getNumMooreOutputs()>0)
  {
    *out << "        q: out " << bit_string << "_vector(" << machine->getNumMooreOutputs()-1
      << " downto 0);" << endl;
  }
  if (machine->getNumOutputs()>0)
  {
    *out << "        o: out " << bit_string << "_vector(" << machine->getNumOutputs()-1
      << " downto 0));" << endl;
  }
// state output *********************
//  *out << "        current_state_out: out std_logic_vector(" << machine->getNumBits()-1
//    << " downto 0));" << endl;
  *out << "end " << mname << ";" << endl << endl;
}




/// Writes the 'architecture' block to the output stream
void ExportVHDL::writeArchitecture()
{
  QString bit_string;

  if (use_std_logic)
    bit_string = "std_logic";
  else
    bit_string = "bit";

  *out << "architecture behaviour of " << mname << " is" << endl << endl;
  if (!state_encoding)
  {
    QList<GState> slist = machine->getSList();
    QString stmp;
    QListIterator<GState> it(slist);
    bool first=TRUE;
    for(; it.current(); ++it)
    {
      GState* st = it.current();
      if (!first)
	stmp += ", ";
      stmp += Utils::noWS(st->getStateName());
      first=FALSE;
    }
    *out << "type state_type is (" << stmp << ");" << endl;
    *out << "signal next_state, current_state : state_type;" << endl << endl;
  }
  else
  {
    *out << "signal current_state, next_state : " << bit_string << "_vector(" <<
      machine->getNumEncodingBits()-1
      << " downto 0);" << endl << endl; 
  }

  *out << "-- comments for syf which comes with alliance : http://www-asim.lip6.fr/alliance"<<endl;
  *out << "-- interpreted as comments by other tools"<<endl;
  *out << "  --pragma CURRENT_STATE current_state" << endl;
  *out << "  --pragma NEXT_STATE next_state" << endl;

  *out << "  --pragma CLOCK clock" << endl << endl;



  *out << "begin" << endl;
// state output ******************
//  *out << "  current_state_out <= current_state;" << endl << endl;
  writeClockProcess();
  writeStateProcess();
  *out << "end behaviour;" << endl;
}




/// Writes the clock process to the output stream
void ExportVHDL::writeClockProcess()
{
  if (synchronous_reset)
  {
    *out << "  process (clock)" << endl;
  }
  else
  {
    *out << "  process (reset, clock)" << endl;
  }

  *out << "  begin" << endl;

  if (!synchronous_reset)
  {
    *out << "    if reset='1' then" << endl;

    GState* stmp = machine->getInitialState();

    if (state_encoding)
    {
      *out << "      current_state <= \"";
      *out << Convert::intToBinStr(stmp->getEncoding(), machine->getNumEncodingBits()) 
	<< "\";" << endl;
    }
    else
    {
      *out << "      current_state <= " << Utils::noWS(stmp->getStateName()) 
	<< ";" << endl;
    }

    *out << "    elsif clock'event and clock='1' then" << endl;
  }
  else
    *out << "    if clock'event and clock='1' then" << endl;

  *out << "      current_state <= next_state;" << endl;
  *out << "    end if;" << endl;
  *out << "  end process;" << endl << endl;
}



/// Writes the input/current_state process to the output stream
void ExportVHDL::writeStateProcess()
{
  GState* s;
  GTransition* t;
  QString tinfoi, tinfoo, sn;
  State* stmp;
  TransitionInfo* tinfo;
  IOInfo* iosingle;
  IOInfo* tioinfo;
  bool first;
  Convert conv;

  //if (synchronous_reset)
  {
    *out << "  process (reset, a, current_state)" << endl;
  }
  /*
  else
  {
    *out << "  process (a, current_state)" << endl;
  }
  */

  *out << "  begin" << endl;

  *out << "    -- default assignments" << endl;
  *out << "--    o <= \"";
  for(int i=0; i<machine->getNumOutputs(); i++)
    *out << "0";
  *out << "\";" << endl;
  *out << "--    next_state <= current_state;" << endl << endl;

  //if (synchronous_reset)
  {
    GState* istate;
    istate = machine->getInitialState();

    *out << "    if reset='1' then " << endl; 

    if (state_encoding)
    {
      *out << "      next_state <= \"";
      *out << Convert::intToBinStr(istate->getEncoding(), machine->getNumEncodingBits()) 
	<< "\";" << endl;
    }
    else
    {
      *out << "      next_state <= " << Utils::noWS(istate->getStateName()) 
	<< ";" << endl;
    }
    *out << "    else" << endl;
  }

  *out << "    case current_state is" << endl;

  QListIterator<GState> is(machine->getSList());

  for(; is.current(); ++is)
  {
    s = is.current();
    sn = s->getStateName();
    sn.replace(QRegExp(" "), "_");
    if (s->countTransitions()>0)
    {
      if (state_encoding)
      {
	  *out << "      when \"" << Convert::intToBinStr(s->getEncoding(),
	      machine->getNumEncodingBits())
	    << "\" =>" << endl;
      }
      else
      {
	  *out << "      when " << Utils::noWS(s->getStateName()) << " =>" << endl;
      }
      if (machine->getNumMooreOutputs() > 0) 
      {
	IOInfo* mout;
	QString smout;
	mout = s->getMooreOutputs();
	smout = mout->convertToBinStr();

	*out << "        q <= \"" 
//	  Convert::intToBinStr(s->getMooreOutputs(), machine->getNumMooreOutputs()) 

	  //<< conv.X10ToBinStr(s->getMooreOutputs())
	  << smout
	  << "\";"<< endl;
      }
    }

    QListIterator<GTransition> it(s->tlist);

    for(; it.current(); ++it)
    {
      t = it.current();
      tinfo = t->getInfo();
      tioinfo = tinfo->getInputInfo();

      if (!t->isDeleted() && t->getEnd())
      {
//	QList<IOInfo> iolist;
//	iolist = tioinfo->getSingles();
	IOInfoList iolist;
	tioinfo->convertToBinList(iolist);
	iolist.setAutoDelete(TRUE);
	
	QListIterator<IOInfo> ioit(iolist);

        *out << "        if a=\"";

	first = TRUE;
	for(; ioit.current(); ++ioit)
	{

	  iosingle = ioit.current();
	  tinfoi = iosingle->convertToBinStr();  

	  if (!tinfoi.isEmpty())
	  {
	    if (!first)
	      *out << "or " << endl << "           a=\"";

	    int slen = tinfoi.length();
	    int numin = machine->getNumInputs();
	    for(int k=slen; k<numin; k++)
	      *out << "0";

	    *out << tinfoi << "\" ";
	    first=FALSE;
	  }
	}

	tinfoo = tinfo->getOutputsStrBin();
	*out << "then" << endl;
	if (!tinfoo.isEmpty())
	{
	  *out << "          o <= \"";

	  int slen = tinfoo.length();
	  int numout = machine->getNumOutputs();
	  for(int l=slen; l<numout; l++)
	    *out << "0";

	  *out << tinfoo << "\";" << endl;
	}
	stmp = t->getEnd();
	if (stmp)
        {
	  sn = stmp->getStateName();
          sn.replace(QRegExp(" "), "_");
        }
	if (stmp!=s)
	{
	  if (state_encoding)
	  {
	    *out << "          next_state <= \"" 
	      << Convert::intToBinStr(stmp->getEncoding(), machine->getNumEncodingBits()) << "\";" << endl;
	  }
	  else
	  {
	    *out << "          next_state <= " <<
	      Utils::noWS(stmp->getStateName()) << ";" << endl;
	  }
	}
	*out << "        end if;" << endl;
      }
    }
  }

  *out << "      when others => null;" << endl;
  *out << "    end case;" << endl;
  if (synchronous_reset)
    *out << "    end if;" << endl;
  *out << "  end process;" << endl << endl;
}

==> qfsm-0.44.tar.gz: qfsm-0.44/src/ExportVHDL.cpp <==
/*
Copyright (C) 2000,2001 Stefan Duffner 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <iostream>
#include <qregexp.h>

#include "ExportVHDL.h"
#include "Machine.h"
#include "TransitionInfo.h"
#include "IOInfo.h"
#include "IOInfoList.h"
#include "Convert.h"
#include "Utils.h"
#include "Options.h"

using namespace std;


ExportVHDL::ExportVHDL(Options* opt)
  	  : Export(opt)
{
}


/// Writes all the relevant data into the tdf file.
void ExportVHDL::doExport()
{
  state_encoding=!options->getVHDLSymbolicStates();
  synchronous_reset=options->getVHDLSyncReset();
  use_std_logic=options->getVHDLStdLogic();

  writeHeader("--", "");
  writeEnvironment();
  writeEntity();
  writeArchitecture();
}


QString ExportVHDL::fileFilter()
{
  return "VHDL (*.vhdl)";
}

/// Writes the 'library' and 'use' lines to the output stream
void ExportVHDL::writeEnvironment()
{
  if (use_std_logic)
  {
    *out << "library ieee;" << endl << endl;
    *out << "use ieee.std_logic_1164.all;" << endl << endl;
  }
}

/// Writes the 'entity' block to the output stream
void ExportVHDL::writeEntity()
{
  QString bit_string;

  if (use_std_logic)
    bit_string = "std_logic";
  else
    bit_string = "bit";

  mname = machine->getName();
  mname.replace(QRegExp(" "), "_");

  *out << "entity " << mname.latin1() << " is" << endl;
  *out << "--  generic (prop_delay : Time := 10 ns);" << endl;
  *out << "  port (clock: in " << bit_string.latin1() << ";" << endl;
  *out << "        reset: in " << bit_string.latin1() << ";" << endl;
  *out << "        a: in " << bit_string.latin1() << "_vector(" << machine->getNumInputs()-1 
    << " downto 0);" << endl;
  if (machine->getNumMooreOutputs()>0)
  {
    *out << "        q: out " << bit_string.latin1() << "_vector(" << machine->getNumMooreOutputs()-1
      << " downto 0);" << endl;
  }
  if (machine->getNumOutputs()>0)
  {
    *out << "        o: out " << bit_string.latin1() << "_vector(" << machine->getNumOutputs()-1
      << " downto 0));" << endl;
  }
// state output *********************
//  *out << "        current_state_out: out std_logic_vector(" << machine->getNumBits()-1
//    << " downto 0));" << endl;
  *out << "end " << mname.latin1() << ";" << endl << endl;
}




/// Writes the 'architecture' block to the output stream
void ExportVHDL::writeArchitecture()
{
  QString bit_string;

  if (use_std_logic)
    bit_string = "std_logic";
  else
    bit_string = "bit";

  *out << "architecture behaviour of " << mname.latin1() << " is" << endl << endl;
  if (!state_encoding)
  {
    QList<GState> slist = machine->getSList();
    QString stmp;
    QListIterator<GState> it(slist);
    bool first=TRUE;
    for(; it.current(); ++it)
    {
      GState* st = it.current();
      if (!first)
	stmp += ", ";
      stmp += Utils::noWS(st->getStateName());
      first=FALSE;
    }
    *out << "type state_type is (" << stmp.latin1() << ");" << endl;
    *out << "signal next_state, current_state : state_type;" << endl << endl;
  }
  else
  {
    *out << "signal current_state, next_state : " << bit_string.latin1() << "_vector(" <<
      machine->getNumEncodingBits()-1
      << " downto 0);" << endl << endl; 
  }

  *out << "-- comments for syf which comes with alliance : http://www-asim.lip6.fr/alliance"<<endl;
  *out << "-- interpreted as comments by other tools"<<endl;
  *out << "  --pragma CURRENT_STATE current_state" << endl;
  *out << "  --pragma NEXT_STATE next_state" << endl;

  *out << "  --pragma CLOCK clock" << endl << endl;



  *out << "begin" << endl;
// state output ******************
//  *out << "  current_state_out <= current_state;" << endl << endl;
  writeClockProcess();
  writeStateProcess();
  *out << "end behaviour;" << endl;
}




/// Writes the clock process to the output stream
void ExportVHDL::writeClockProcess()
{
  if (synchronous_reset)
  {
    *out << "  process (clock)" << endl;
  }
  else
  {
    *out << "  process (reset, clock)" << endl;
  }

  *out << "  begin" << endl;

  if (!synchronous_reset)
  {
    *out << "    if reset='1' then" << endl;

    GState* stmp = machine->getInitialState();

    if (state_encoding)
    {
      *out << "      current_state <= \"";
      *out << Convert::intToBinStr(stmp->getEncoding(), machine->getNumEncodingBits()).latin1() 
	<< "\";" << endl;
    }
    else
    {
      *out << "      current_state <= " <<
	Utils::noWS(stmp->getStateName()).latin1() 
	<< ";" << endl;
    }

    *out << "    elsif clock'event and clock='1' then" << endl;
  }
  else
    *out << "    if clock'event and clock='1' then" << endl;

  *out << "      current_state <= next_state;" << endl;
  *out << "    end if;" << endl;
  *out << "  end process;" << endl << endl;
}



/// Writes the input/current_state process to the output stream
void ExportVHDL::writeStateProcess()
{
  GState* s;
  GTransition* t;
  QString tinfoi, tinfoo, sn;
  State* stmp;
  TransitionInfo* tinfo;
  IOInfo* iosingle;
  IOInfo* tioinfo;
  bool first;
  Convert conv;

  //if (synchronous_reset)
  {
    *out << "  process (reset, a, current_state)" << endl;
  }
  /*
  else
  {
    *out << "  process (a, current_state)" << endl;
  }
  */

  *out << "  begin" << endl;

  *out << "    -- default assignments" << endl;
  *out << "--    o <= \"";
  for(int i=0; i<machine->getNumOutputs(); i++)
    *out << "0";
  *out << "\";" << endl;
  *out << "--    next_state <= current_state;" << endl << endl;

  //if (synchronous_reset)
  {
    GState* istate;
    istate = machine->getInitialState();

    *out << "    if reset='1' then " << endl; 

    if (state_encoding)
    {
      *out << "      next_state <= \"";
      *out << Convert::intToBinStr(istate->getEncoding(),
	  machine->getNumEncodingBits()).latin1() 
	<< "\";" << endl;
    }
    else
    {
      *out << "      next_state <= " <<
	Utils::noWS(istate->getStateName()).latin1() 
	<< ";" << endl;
    }
    *out << "    else" << endl;
  }

  *out << "    case current_state is" << endl;

  QListIterator<GState> is(machine->getSList());

  for(; is.current(); ++is)
  {
    s = is.current();
    sn = s->getStateName();
    sn.replace(QRegExp(" "), "_");
    if (s->countTransitions()>0)
    {
      if (state_encoding)
      {
	  *out << "      when \"" <<
	    Convert::intToBinStr(s->getEncoding(),
	      machine->getNumEncodingBits()).latin1()
	    << "\" =>" << endl;
      }
      else
      {
	  *out << "      when " << Utils::noWS(s->getStateName()).latin1() << " =>" << endl;
      }
      if (machine->getNumMooreOutputs() > 0) 
      {
	IOInfo* mout;
	QString smout;
	mout = s->getMooreOutputs();
	smout = mout->convertToBinStr();

	if (!smout.isEmpty())
	  *out << "        q <= \"" << smout.latin1() << "\";"<< endl;
      }
    }

    QListIterator<GTransition> it(s->tlist);

    for(; it.current(); ++it)
    {
      t = it.current();
      tinfo = t->getInfo();
      tioinfo = tinfo->getInputInfo();

      if (!t->isDeleted() && t->getEnd())
      {
//	QList<IOInfo> iolist;
//	iolist = tioinfo->getSingles();
	IOInfoList iolist;
	tioinfo->convertToBinList(iolist, FALSE);
	iolist.setAutoDelete(TRUE);
	
	QListIterator<IOInfo> ioit(iolist);

        *out << "        if a";
	if (tioinfo->isInverted())
	  *out << "/=";
	else
	  *out << "=";
	*out << "\"";

	first = TRUE;
	for(; ioit.current(); ++ioit)
	{

	  iosingle = ioit.current();
	  tinfoi = iosingle->convertToBinStr();  
	  tinfoi.replace(QRegExp("x"), "-");

	  if (!tinfoi.isEmpty())
	  {
	    if (!first)
	    {
	      if (tioinfo->isInverted())
		*out << "and ";
	      else
		*out << "or ";
	      *out << endl << "           a";
	      if (tioinfo->isInverted())
		*out << "/=";
	      else
		*out << "=";
	      *out << "\"";
	    }

	    int slen = tinfoi.length();
	    int numin = machine->getNumInputs();
	    for(int k=slen; k<numin; k++)
	      *out << "0";

	    *out << tinfoi.latin1() << "\" ";
	    first=FALSE;
	  }
	}

	tinfoo = tinfo->getOutputsStrBin();
	*out << "then" << endl;
	if (!tinfoo.isEmpty())
	{
	  *out << "          o <= \"";

	  int slen = tinfoo.length();
	  int numout = machine->getNumOutputs();
	  for(int l=slen; l<numout; l++)
	    *out << "0";

	  *out << tinfoo.latin1() << "\";" << endl;
	}
	stmp = t->getEnd();
	if (stmp)
        {
	  sn = stmp->getStateName();
          sn.replace(QRegExp(" "), "_");
        }
	if (stmp!=s)
	{
	  if (state_encoding)
	  {
	    *out << "          next_state <= \"" 
	      << Convert::intToBinStr(stmp->getEncoding(),
		  machine->getNumEncodingBits()).latin1() << "\";" << endl;
	  }
	  else
	  {
	    *out << "          next_state <= " <<
	      Utils::noWS(stmp->getStateName()).latin1() << ";" << endl;
	  }
	}
	*out << "        end if;" << endl;
      }
    }
  }

  *out << "      when others => null;" << endl;
  *out << "    end case;" << endl;
  if (synchronous_reset)
    *out << "    end if;" << endl;
  *out << "  end process;" << endl << endl;
}
